@@ -1,6 +1,21 @@
 from django.conf import settings
+
 from adminlte2_templates import constants
 
+try:
+    # Python 3
+    from urllib.parse import urlencode
+except ImportError:
+    # Python 2.7
+    from urllib import urlencode
+
+try:
+    # Supports >=Django 2.0
+    from django.shortcuts import reverse
+except ImportError:
+    # Supports <=Django 1.1
+    from django.core.urlresolvers import reverse
+
 
 def get_settings(variable, django_setting=None):
     """
